''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' NAME:		check_complus.vbs
' VERSION:	0.1
' AUTHOR:	Alexander Rudolf (alexander.rudolf@saxsys.de)
'
' COMMENT:	Script for checking COM+ applications and their process performance counter
'		for use with Nagios and NSClient++
'
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Option explicit
Dim strAppID
Dim strAppPID
Dim strCOMAppName
Dim strCounter
Dim strWMIQuery
Dim strHelpCounter
Dim objArgs
Dim objApp
Dim objApplications
Dim objApplicationInstances
Dim objCatalog
Dim objItem
Dim objWMIService
Dim colItems
Dim arrHelpCounter
Dim valResult
Dim valMaxWarn
Dim valMaxCrit
Dim i
Const wbemFlagReturnImmediately = &h10
Const wbemFlagForwardOnly = &h20

Set objArgs = WScript.Arguments

If objArgs.Count = 0 Then
	Wscript.StdOut.WriteLine "CRITICAL - Bad number of arguments"
	WScript.Quit(2)
End if

if objArgs(0) = "--help" Then
	Wscript.StdOut.WriteLine ""
	Wscript.StdOut.WriteLine "CHECK_COMPLUS Usage"
	Wscript.StdOut.WriteLine ""
	Wscript.StdOut.WriteLine "check_complus.vbs APP application,[application]"
	Wscript.StdOut.WriteLine "check_complus.vbs COUNTER application counter MaxWarn MaxCrit"
	Wscript.StdOut.WriteLine ""
	Wscript.StdOut.WriteLine vbTab & "APP               check if specifies applications are running"
	Wscript.StdOut.WriteLine vbTab & "COUNTER           return performance counter data"
	Wscript.StdOut.WriteLine ""
	Wscript.StdOut.WriteLine vbTab & "application       COM+ application names to be monitored"
	Wscript.StdOut.WriteLine vbTab & "counter           performance counter to be monitored"
	Wscript.StdOut.WriteLine vbTab & "MaxWarn           warning threshold"
	Wscript.StdOut.WriteLine vbTab & "MaxCrit           critical threshold"
	Wscript.StdOut.WriteLine ""
	Wscript.StdOut.WriteLine "List items are comma separated. Counter name must be without spaces, i.e."
	Wscript.StdOut.WriteLine "'PoolNonpagedBytes'. Use --helpcounter to get a list of supported counters."
	Wscript.StdOut.WriteLine ""
	WScript.Quit(0)
End if

if objArgs(0) = "--helpcounter" Then
	Wscript.StdOut.WriteLine ""
	Wscript.StdOut.WriteLine "CHECK_COMPLUS Supported performance counters"
	Wscript.StdOut.WriteLine ""
	Wscript.StdOut.WriteLine "Please be aware that some counters are non numerical and therefore unusable."
	Wscript.StdOut.WriteLine ""

	arrHelpCounter = array ("Caption","CreatingProcessID","Description","ElapsedTime","Frequency_Object","Frequency_PerfTime","Frequency_Sys100NS" _
	,"HandleCount","IDProcess","IODataBytesPersec","IODataOperationsPersec","IOOtherBytesPersec","IOOtherOperationsPersec","IOReadBytesPersec" _
	,"IOReadOperationsPersec","IOWriteBytesPersec","IOWriteOperationsPersec","Name","PageFaultsPersec","PageFileBytes","PageFileBytesPeak" _
	,"PercentPrivilegedTime","PercentProcessorTime","PercentUserTime","PoolNonpagedBytes","PoolPagedBytes","PriorityBase","PrivateBytes" _
	,"ThreadCount","Timestamp_Object","Timestamp_PerfTime","Timestamp_Sys100NS","VirtualBytes","VirtualBytesPeak","WorkingSet","WorkingSetPeak")
	
	for each strHelpCounter in arrHelpCounter
		Wscript.StdOut.WriteLine vbTab & "* " & strHelpCounter
	Next
	Wscript.StdOut.WriteLine ""
	WScript.Quit(0)
End if


If objArgs.Count < 2 Then
	Wscript.StdOut.WriteLine "CRITICAL - Bad number of arguments"
	WScript.Quit(2)
End if

Set objCatalog = CreateObject("ComAdmin.COMAdminCatalog")
Set objApplications = objCatalog.GetCollection("Applications")
Set objApplicationInstances = objCatalog.GetCollection("ApplicationInstances")

objApplications.Populate
objApplicationInstances.Populate

Select Case objArgs(0)

	Case "APP"
		Wscript.StdOut.WriteLine "no yet implemented"
		wscript.quit(0)

	Case "COUNTER"
		If objArgs.Count < 5 Then
			Wscript.StdOut.WriteLine "CRITICAL - Missing arguments"
			WScript.Quit(2)
		End if

		strCOMAppName = objArgs(1)
		strCounter = objArgs(2)
		valMaxWarn = Cdbl(objArgs(3))
		valMaxCrit = Cdbl(objArgs(4))
		If valMaxCrit < valMaxWarn Then
			Wscript.StdOut.WriteLine "CRITICAL - critical below warning level (" & valMaxCrit & " < " & valMaxWarn & ")"
			Wscript.quit(2)
		End If

		for i = 0 to objApplications.Count - 1
			Set objApp = objApplications.item(i)
			if objApp.Value("Name") = strCOMAppName Then
				strAppID = Cstr(objApp.Value("ID"))
			End If
		Next

		If strAppID = "" Then
			Wscript.StdOut.WriteLine "CRITICAL - COM+ application '" + strCOMAppName + "' not found."
			WScript.Quit(2)
		End if

		for i = 0 to objApplicationInstances.Count - 1
			Set objApp = objApplicationInstances.item(i)
			If objApp.Value("Application") = strAppID Then
				strAppPID = objApp.Value("ProcessID")
			End If
		Next

		If strAppPID = "" Then
			Wscript.StdOut.WriteLine "CRITICAL - COM+ application '" + strCOMAppName + "' seems not be running."
			WScript.Quit(2)
		End if

		Set objWMIService = GetObject("winmgmts:\\.\root\CIMV2")
		strWMIQuery = "SELECT * FROM Win32_PerfFormattedData_PerfProc_Process WHERE IDProcess = " & strAppPID
		Set colItems = objWMIService.ExecQuery(strWMIQuery, "WQL", wbemFlagReturnImmediately + wbemFlagForwardOnly)

		For Each objItem In colItems
		Select case strCounter
			Case "Caption" valResult = Cdbl(objItem.Caption)
			Case "CreatingProcessID" valResult = Cdbl(objItem.CreatingProcessID)
			Case "Description" valResult = Cdbl(objItem.Description)
			Case "ElapsedTime" valResult = Cdbl(objItem.ElapsedTime)
			Case "Frequency_Object" valResult = Cdbl(objItem.Frequency_Object)
			Case "Frequency_PerfTime" valResult = Cdbl(objItem.Frequency_PerfTime)
			Case "Frequency_Sys100NS" valResult = Cdbl(objItem.Frequency_Sys100NS)
			Case "HandleCount" valResult = Cdbl(objItem.HandleCount)
			Case "IDProcess" valResult = Cdbl(objItem.IDProcess)
			Case "IODataBytesPersec" valResult = Cdbl(objItem.IODataBytesPersec)
			Case "IODataOperationsPersec" valResult = Cdbl(objItem.IODataOperationsPersec)
			Case "IOOtherBytesPersec" valResult = Cdbl(objItem.IOOtherBytesPersec)
			Case "IOOtherOperationsPersec" valResult = Cdbl(objItem.IOOtherOperationsPersec)
			Case "IOReadBytesPersec" valResult = Cdbl(objItem.IOReadBytesPersec)
			Case "IOReadOperationsPersec" valResult = Cdbl(objItem.IOReadOperationsPersec)
			Case "IOWriteBytesPersec" valResult = Cdbl(objItem.IOWriteBytesPersec)
			Case "IOWriteOperationsPersec" valResult = Cdbl(objItem.IOWriteOperationsPersec)
			Case "Name" valResult = Cdbl(objItem.Name)
			Case "PageFaultsPersec" valResult = Cdbl(objItem.PageFaultsPersec)
			Case "PageFileBytes" valResult = Cdbl(objItem.PageFileBytes)
			Case "PageFileBytesPeak" valResult = Cdbl(objItem.PageFileBytesPeak)
			Case "PercentPrivilegedTime" valResult = Cdbl(objItem.PercentPrivilegedTime)
			Case "PercentProcessorTime" valResult = Cdbl(objItem.PercentProcessorTime)
			Case "PercentUserTime" valResult = Cdbl(objItem.PercentUserTime)
			Case "PoolNonpagedBytes" valResult = Cdbl(objItem.PoolNonpagedBytes)
			Case "PoolPagedBytes" valResult = Cdbl(objItem.PoolPagedBytes)
			Case "PriorityBase" valResult = Cdbl(objItem.PriorityBase)
			Case "PrivateBytes" valResult = Cdbl(objItem.PrivateBytes)
			Case "ThreadCount" valResult = Cdbl(objItem.ThreadCount)
			Case "Timestamp_Object" valResult = Cdbl(objItem.Timestamp_Object)
			Case "Timestamp_PerfTime" valResult = Cdbl(objItem.Timestamp_PerfTime)
			Case "Timestamp_Sys100NS" valResult = Cdbl(objItem.Timestamp_Sys100NS)
			Case "VirtualBytes" valResult = Cdbl(objItem.VirtualBytes)
			Case "VirtualBytesPeak" valResult = Cdbl(objItem.VirtualBytesPeak)
			Case "WorkingSet" valResult = Cdbl(objItem.WorkingSet)
			Case "WorkingSetPeak" valResult = Cdbl(objItem.WorkingSetPeak)
			Case Else
		End Select
		Next

		If valResult >= valMaxCrit Then
			Wscript.StdOut.WriteLine strCounter & " CRITICAL - " & strCOMAppName & " (PID " & strAppPID & "): " & valResult
			Wscript.StdOut.WriteLine strAppID
			Wscript.StdOut.WriteLine "|" & strCounter & "=" & valResult & ";" & valMaxWarn & ";" & valMaxCrit
			Wscript.quit(2)
		End If
		If valResult >= valMaxWarn Then
			Wscript.StdOut.WriteLine strCounter & " WARNING - " & strCOMAppName & " (PID " & strAppPID & "): " & valResult
			Wscript.StdOut.WriteLine strAppID
			Wscript.StdOut.WriteLine "|" & strCounter & "=" & valResult & ";" & valMaxWarn & ";" & valMaxCrit
			Wscript.quit(1)
		End If
		Wscript.StdOut.WriteLine strCounter & " OK - " & strCOMAppName & " (PID " & strAppPID & "): " & valResult
		Wscript.StdOut.WriteLine strAppID
		Wscript.StdOut.WriteLine "|" & strCounter & "=" & valResult & ";" & valMaxWarn & ";" & valMaxCrit
		Wscript.quit(0)

	Case Else
			Wscript.StdOut.WriteLine "CRITICAL - bad argument: " & objArgs(0)
			WScript.Quit(2)

End Select